//xulunit.js

//Metaphor (Revisar):
// A TestSuite is a set of TestCases
// Each TestCase is an object with a Test Function which calls, among other commands, some test functions.
// Test functions are assertions (begin with assert...) that check a condition.
// When a TestCase is run, the function is called and all tests (assertions) are checked.
// Results for each Test are recorded on the TestCase 
// Results for each TestCase are recorded on the TestSuite

//Global Variables/*{{{*/
var suiteOnExecution = null;/*}}}*/

//Constant values/*{{{*/
const XULUNIT_PASS = 1;
const XULUNIT_FAIL = 2;
const XULUNIT_BREAK = 3;
const XULUNIT_UNKNOWN = 4;
const XULUNIT_ASSERT = 5;
const XULUNIT_ASSERTTRUE = 6;
const XULUNIT_ASSERTFALSE = 7;
const XULUNIT_ASSERTEQUALS = 8;
const XULUNIT_ASSERTNOTEQUALS = 9;
const XULUNIT_ASSERTNULL = 10;
const XULUNIT_ASSERTNOTNULL = 11;
const XULUNIT_ASSERTUNDEFINED = 12;
const XULUNIT_ASSERTNOTUNDEFINED = 13;
const XULUNIT_ASSERTNAN = 14;
const XULUNIT_ASSERTNOTNAN = 15;
const XULUNIT_TESTSUITE = 16;
const XULUNIT_TESTCASE = 17;
/*}}}*/

//Class TestSuite
function TestSuite(name){/*{{{*/
//Creates a new TestSuite
  this.type = XULUNIT_TESTSUITE;
  this.name = name;
  this.result = XULUNIT_UNKNOWN;
  this.testCase = new Array();
  this.testCaseOnExecution = "";

  this.add = addTestSuite;
  this.run = runTestSuite;
  this.prepare = prepareTestSuite;
  this.execute = executeTestSuite;
  this.close = closeTestSuite;
  this.getResult = getResultTestSuite;

  function addTestSuite(testCase){/*{{{*/
  //Adds a TestCase to the TestSuite
    this.testCase[this.testCase.length] = testCase;
  }/*}}}*/

  function runTestSuite(){/*{{{*/
  //Executes TestCases in the TestSuite and records results
  //by calling prepare, execute and close
    suiteOnExecution = this;
    this.prepare();
    this.execute();
    this.close();
  }/*}}}*/

  function prepareTestSuite(){/*{{{*/
  //Does any action needed to prepare execution of TestSuite

    //To be overriden
  
  }/*}}}*/

  function executeTestSuite(){/*{{{*/
  //Actually executes TestCases in TestSuite
    for (var i=0; i<this.testCase.length; i++){
      this.testCaseOnExecution = this.testCase[i];
      try{this.testCaseOnExecution.run();}
      catch(ex){
	this.testCaseOnExecution.result = XULUNIT_BREAK;
	this.testCaseOnExecution.breakMessage = ex;
      }
    }
  }/*}}}*/

  function closeTestSuite(){/*{{{*/
  //Does any action needed to close the execution of TestSuite

    //To be overriden
  
  }/*}}}*/

  function getResultTestSuite(){/*{{{*/
  //Returns a boolean with the result of the overall TestCases in TestSuite
    if (this.testCase.length > 0) this.result = this.testCase[0].getResult();
    for (var i=1; i<this.testCase.length; i++)
      if (this.testCase[i].getResult() != XULUNIT_PASS) this.result = XULUNIT_FAIL;

    return this.result;
  }/*}}}*/

}//TestSuite Class/*}}}*/

//Class TestCase
function TestCase(name, testFunction){/*{{{*/
//Creates a TestCase
  this.type = XULUNIT_TESTCASE;
  this.name = name;
  this.testFunction = testFunction;
  this.result = XULUNIT_UNKNOWN;
  this.test = new Array();
  this.testOnExecution = null;

  this.run = runTestCase;
  this.getResult = getResultTestCase;
  this.launchTest = launchTestTestCase;

  function runTestCase(){/*{{{*/
  //Runs the TestCase and records the result
    this.testFunction();
  }/*}}}*/
   
  function getResultTestCase(){/*{{{*/
  //Sets and returns the result of the TestCase
    if (this.test.length > 0) this.result = this.test[0].result;
    for (var i=1; i<this.test.length; i++)
      if (this.test[i].result != XULUNIT_PASS) this.result = XULUNIT_FAIL;

    return this.result;
  }/*}}}*/
  
  function launchTestTestCase(testType, comment){/*{{{*/
  //Called when a test is launched to record its information
  //Returns a pointer to the new test
    currentTest = new Test(testType, comment);
    this.test[this.test.length] = currentTest;
    testOnExecution = currentTest;

    return currentTest;
  }/*}}}*/

}//TestCase class/*}}}*/

//Class Test
function Test(testType, comment){/*{{{*/
  this.type = testType;
  this.comment = comment;
  this.result = XULUNIT_UNKNOWN;
  this.breakMessage = "";
  
  this.passes = passesTest;
  this.fails = failsTest;
  this.breaks = breaksTest;
  
  function passesTest(){/*{{{*/
  //Called when a test passes to record its results
    this.result = XULUNIT_PASS;
  }/*}}}*/

  function failsTest(){/*{{{*/
  //Called when a test fails to record its results
    this.result = XULUNIT_FAIL;
  }/*}}}*/

  function breaksTest(message){/*{{{*/
  //Called when a test breaks to record its results
    this.result = XULUNIT_BREAK;
    this.breakMessage = message;
  }/*}}}*/
}/*}}}*/

//Test Functions
function assert(comment, toEval){/*{{{*/
//Asserts toEval expression equals true
  _assert(XULUNIT_ASSERT, false, comment, true, toEval);
}/*}}}*/

function assertTrue(comment, toEval){/*{{{*/
//Asserts toEval expression equals true
  _assert(XULUNIT_ASSERTTRUE, false, comment, true, toEval);
}/*}}}*/

function assertFalse(comment, toEval){/*{{{*/
//Asserts toEval expression equals false
  _assert(XULUNIT_ASSERTFALSE, false, comment, false, toEval);
}/*}}}*/

function assertEquals(comment, value, toEval){/*{{{*/
//Asserts toEval expression equals value
  _assert(XULUNIT_ASSERTEQUALS, false, comment, value, toEval);
}/*}}}*/

function assertNotEquals(comment, value, toEval){/*{{{*/
//Asserts toEval expression does not equals value
  _assert(XULUNIT_ASSERTNOTEQUALS, true, comment, value, toEval);
}/*}}}*/

function assertNull(comment, toEval){/*{{{*/
//Asserts toEval expression is null
  _assert(XULUNIT_ASSERTNULL, false, comment, null, toEval);
}/*}}}*/

function assertNotNull(comment, toEval){/*{{{*/
//Asserts toEval expression is not null
  _assert(XULUNIT_ASSERTNOTNULL, true, comment, null, toEval);
}/*}}}*/

function assertUndefined(comment, toEval){/*{{{*/
//Asserts toEval expression is undefined
  _assert(XULUNIT_ASSERTUNDEFINED, false, comment, undefined, toEval);
}/*}}}*/

function assertNotUndefined(comment, toEval){/*{{{*/
//Asserts toEval expression is not undefined
  _assert(XULUNIT_ASSERTNOTUNDEFINED, true, comment, undefined, toEval);
}/*}}}*/

function assertNaN(comment, toEval){/*{{{*/
//Asserts toEval expression is Nan (Not a Number)
  _assert(XULUNIT_ASSERTNAN, false, comment, true, isNaN(toEval));
}/*}}}*/

function assertNotNaN(comment, toEval){/*{{{*/
//Asserts toEval expression is not Nan (Not a Number). I.e. it is a number
  _assert(XULUNIT_ASSERTNAN, true, comment, true, isNaN(toEval));
}/*}}}*/

function _assert(type, negate, comment, value, toEval){/*{{{*/
//Real assertion function for any case
  currentTest = suiteOnExecution.testCaseOnExecution.launchTest(type, comment);
  try{
    if (!negate){
      if (toEval == value) currentTest.passes();
      else currentTest.fails();
    }
    else{
      if (toEval != value) currentTest.passes();
      else currentTest.fails();
    }
  }
  catch(ex){
    currentTest.breaks(ex);
  }
}/*}}}*/
